"use client";

import { useMemo } from "react";

import { yupResolver } from "@hookform/resolvers/yup";
import { UseFormProps, useForm } from "react-hook-form";

import Yup from "@/components/FormBuilder/utils/yup";

export type FormField = {
	yup: Yup.Schema;
};

export type FormFields = Record<string, FormField>;

export type FormSchema<T extends FormFields> = {
	[K in keyof T]: Yup.InferType<T[K]["yup"]>;
};

export type YupSchema<T extends FormFields> = Yup.ObjectSchema<FormSchema<T>>;

export type FormValues<T extends FormFields> = Yup.InferType<YupSchema<T>>;

const useFormBuilder = <T extends FormFields>(
	fields: T,
	options?: Omit<UseFormProps<FormValues<T>>, "resolver">
) => {
	const schema = useMemo(
		() =>
			Yup.object({
				...Object.entries(fields).reduce(
					(acc, [name, field]) => ({
						...acc,
						[name]: field.yup,
					}),
					{}
				),
			}) as YupSchema<T>,
		[fields]
	);

	const resolver = useMemo(() => yupResolver(schema), [schema]);

	const formHooks = useForm<FormValues<T>>({
		resolver,
		...options,
	});

	return formHooks;
};

export default useFormBuilder;
